/////////////////////////////////////////////////////////////////////////////
// AsmThread.cpp
//
// Copyright (C) 2000 Jeremy Goetsch (jeremy@acz.org)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#include "stdafx.h"
#include "Asmide.h"
#include "AsmThread.h"
#include "TwoPassAsm.h"
#include "OnePassAsm.h"
#include "AsmOutputStream.h"

/////////////////////////////////////////////////////////////////////////////
// SingleAsmThreadProc
//  Worker thread procedure for assembling a single file
//  pParam points to a SingleAsmThreadInfo structure with members:
//    pAsm - pointer to CAssembler or derived class to invoke
//    lpszPath - path and filename of file to assemble
//    hwndNotify - handle of wnd to receive assembly begin and end messages
//    hwndOutputEdit - handle of control to receive assembly output
//
//  The structure, as well as the pAsm and lpszPath members,
//  are deleted automatically by the procedure and must be allocated
//  on the heap by the calling thread.

UINT SingleAsmThreadProc( SingleAsmThreadInfo* pInfo )
{
	ASSERT(pInfo != NULL);
	ASSERT(pInfo->pAsm != NULL);
	ASSERT(pInfo->lpszPath != NULL);

	//HANDLE hEventAbort = CreateEvent(NULL, FALSE, FALSE, NULL);	// auto reset, initially reset
	//::SendMessage(pInfo->hwndNotify, WM_USER_ASM_BEGIN, 0, (LPARAM)hEventAbort);

	SetCurrentDirectory(GetDirFromPath(pInfo->lpszPath));

	if (pInfo->pAsm->LoadTabFile( GetOptions()->m_asmTabFile))
		pInfo->pAsm->Build( pInfo->lpszPath);
	pInfo->pAsm->Finish();

	UINT nResult = pInfo->pAsm->GetStatus();
	delete pInfo->pAsm;
	//delete pInfo->lpszPath;
	//::SendMessage(pInfo->hwndNotify, WM_USER_ASM_DONE, 0, 0L);
	//CloseHandle(hEventAbort);
	delete pInfo;
	return nResult;	// return assembly status as thread result
}

/////////////////////////////////////////////////////////////////////////////
// ProjectAsmThreadProc
//  Worker thread procedure for assembling a project
//  pParam points to a ProjectAsmThreadInfo structure with members:
//    pBuildType - pointer to CBuildType to build
//    lpszTabFile - tab file to use
//    nAsmFlags - flags to pass to assembler
//    hwndNotify - handle of wnd to receive assembly begin and end messages
//    hwndOutputEdit - handle of control to receive assembly output
//
//  The structure is deleted automatically by the procedure and must be
//  allocated on the heap by the calling thread. None of the data given
//  can be changed by the calling thread until the build is complete.
/*
#define CONSOLE_BUFFER_SIZE	1024

UINT ProjectAsmThreadProc( LPVOID pParam)
{
	ProjectAsmThreadInfo *pInfo = (ProjectAsmThreadInfo *)pParam;
	ASSERT(pInfo->pBuildType != NULL);
	ASSERT(IsWindow(pInfo->hwndNotify));
	ASSERT(IsWindow(pInfo->hwndOutputEdit));

	HANDLE hEventAbort = CreateEvent(NULL, FALSE, FALSE, NULL);	// auto reset, initially reset
	::SendMessage(pInfo->hwndNotify, WM_USER_ASM_BEGIN, 0, (LPARAM)hEventAbort);

	CBuildStep bstep;
	CAssembler *pAsm = NULL;
	CAsmOutputStream asmout(pInfo->hwndOutputEdit, hEventAbort);
	BOOL bStopBuild = FALSE;
	CString szOldFilename;
	CFileStatus fstat;

	// structures and handles for creating cmd line processes
	STARTUPINFO sInfo;
	PROCESS_INFORMATION procInfo;
	SECURITY_ATTRIBUTES saAttr;
	HANDLE hReadOutput, hWriteOutput, hReadOutputTmp, hWriteError;
	HANDLE hReadInput, hWriteInput, hWriteInputTmp;
	LPTSTR lpszCmdLine;
	LPCSTR lpszInitialDir;
	ZeroMemory(&sInfo, sizeof(STARTUPINFO));
	sInfo.cb = sizeof(STARTUPINFO);
	saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
	saAttr.bInheritHandle = TRUE;
	saAttr.lpSecurityDescriptor = NULL;

	POSITION pos = pInfo->pBuildType->m_buildsteps.GetHeadPosition();
	while (pos != NULL && !bStopBuild) {
		if (pAsm)
			szOldFilename = bstep.szFilename;
		bstep = pInfo->pBuildType->m_buildsteps.GetNext(pos);

		CString fn;
		CAssembler *pOldAsm = pAsm;
		LPCSTR lpszDir;
		switch (bstep.nType) {
			case CBuildStep::typeAssemble:
				pAsm = new CPluginAssembler(pInfo->nAsmFlags, bstep.szPlugin.Left(bstep.szPlugin.ReverseFind('.')),
					pInfo->pBuildType->m_szDefines);
				break;
			case CBuildStep::typeHexListing:
				pAsm = new CHexAssembler(pInfo->nAsmFlags, pInfo->pBuildType->m_szDefines);
				break;
			case CBuildStep::typeSymTable:
				pAsm = new CSymTableAssembler(pInfo->nAsmFlags, pInfo->pBuildType->m_szDefines);
				break;
			case CBuildStep::typeExternal:
				sInfo.dwFlags = 0;

				lpszCmdLine = bstep.szFilename.GetBuffer(1);
				if (bstep.szExtDir.IsEmpty())
					lpszInitialDir = NULL;
				else
					lpszInitialDir = bstep.szExtDir;
				if (!CreateProcess( NULL,			// application name
									lpszCmdLine,	// command line
									NULL,			// process attributes
									NULL,			// thread attributes
									FALSE,			// inherit parent handles
									0,				// creation flags
									NULL,				// environment block
									lpszInitialDir,		// initial directory
									&sInfo, &procInfo)	// startup info and process info
					&& GetLastError() == ERROR_FILE_NOT_FOUND)
				{
					// try again from exe path
					lpszDir = SetDirDefault();
					if (!CreateProcess( NULL, lpszCmdLine, NULL, NULL, FALSE, 0, NULL, lpszInitialDir,
										&sInfo, &procInfo))
					{
						TRACE1("CreateProcess failed with code 0x%x\n", GetLastError());
						AfxMessageBox( IDP_ERROR_EXEC, MB_ICONEXCLAMATION);
					}
					SetCurrentDirectory(lpszDir);
				}
				bstep.szFilename.ReleaseBuffer();
				continue;

			case CBuildStep::typeCommandLine:
				// create anonymous pipe for standard output
				CreatePipe(&hReadOutputTmp, &hWriteOutput, &saAttr, 0);
				// duplicate pipe for std error output
				DuplicateHandle(GetCurrentProcess(), hWriteOutput, GetCurrentProcess(), &hWriteError,
					0, TRUE, DUPLICATE_SAME_ACCESS);
				// create noninheritable duplicate of read handle
				DuplicateHandle(GetCurrentProcess(), hReadOutputTmp, GetCurrentProcess(), &hReadOutput,
					0, FALSE, DUPLICATE_SAME_ACCESS);

				// create anonymous pipe for standard input
				CreatePipe(&hReadInput, &hWriteInputTmp, &saAttr, 32);
				SetStdHandle(STD_INPUT_HANDLE, hReadInput);
				// create noninheritable duplicate of write handle
				DuplicateHandle(GetCurrentProcess(), hWriteInputTmp, GetCurrentProcess(), &hWriteInput,
					0, FALSE, DUPLICATE_SAME_ACCESS);

				// close inheritable copies of handles we don't want inherited
				CloseHandle(hReadOutputTmp);
				CloseHandle(hWriteInputTmp);

				sInfo.dwFlags = STARTF_USESTDHANDLES | STARTF_USESHOWWINDOW;
				sInfo.wShowWindow = SW_HIDE;
				sInfo.hStdOutput = hWriteOutput;
				sInfo.hStdError = hWriteError;
				sInfo.hStdInput = hReadInput;

				lpszCmdLine = bstep.szFilename.GetBuffer(1);
				if (bstep.szExtDir.IsEmpty())
					lpszInitialDir = NULL;
				else
					lpszInitialDir = bstep.szExtDir;
				// create child process which will inherit std handles of parent
				if (!CreateProcess( NULL,			// application name
									lpszCmdLine,	// command line
									NULL,			// process attributes
									NULL,			// thread attributes
									TRUE,			// inherit parent handles
									0,				// creation flags
									NULL,				// environment block
									lpszInitialDir,		// initial directory
									&sInfo, &procInfo)	// startup info and process info
					&& GetLastError() == ERROR_FILE_NOT_FOUND)
				{
					// try again from exe path
					lpszDir = SetDirDefault();
					if (!CreateProcess( NULL, lpszCmdLine, NULL, NULL, TRUE, 0, NULL, lpszInitialDir,
										&sInfo, &procInfo))
					{
						CloseHandle(hWriteOutput);
						CloseHandle(hWriteError);
						CloseHandle(hReadInput);
						TRACE1("CreateProcess failed with code 0x%x\n", GetLastError());
						AfxMessageBox( IDP_ERROR_EXEC, MB_ICONEXCLAMATION);
					}
					SetCurrentDirectory(lpszDir);
				}
				else {
					// these handles no longer needed by parent
					CloseHandle(hWriteOutput);
					CloseHandle(hWriteError);
					CloseHandle(hReadInput);

					char buffer[CONSOLE_BUFFER_SIZE];
					DWORD nBytes, nRead;
					// This thread now just checks for output from the spawned process
					// until the process ends or the build is aborted.
					// WaitForMultipleObjects causes us to sleep for 200 msec between
					// checks on the output pipe so as not to waste too much CPU time.
					DWORD dwResult;
					HANDLE wait_objects[] = { hEventAbort, procInfo.hProcess };
					while ((dwResult = WaitForMultipleObjects(2, wait_objects, FALSE, 200)) == WAIT_TIMEOUT) {
						PeekNamedPipe(hReadOutput, NULL, 0, NULL, &nBytes, NULL);
						if (nBytes > 0) {
							ReadFile(hReadOutput, buffer, CONSOLE_BUFFER_SIZE-1, &nRead, NULL);
							buffer[nRead] = '\0';
							asmout.PrintInfo(buffer);
						}
					}
					if (dwResult == WAIT_OBJECT_0) {	// hEventAbort signalled
						TerminateProcess(procInfo.hProcess, -1);
						asmout.PrintInfo("** Command line process terminated by user **\n");
						bStopBuild = TRUE;
					}
					// get output remaining in pipe after process ends
					PeekNamedPipe(hReadOutput, NULL, 0, NULL, &nBytes, NULL);
					if (nBytes > 0) {
						ReadFile(hReadOutput, buffer, CONSOLE_BUFFER_SIZE-1, &nRead, NULL);
						buffer[nRead] = '\0';
						asmout.PrintInfo(buffer);
					}
				}
				bstep.szFilename.ReleaseBuffer();
				CloseHandle(hReadOutput);
				CloseHandle(hWriteInput);
				continue;
			default:
				ASSERT(0);
		}

		if (pOldAsm) {
			if (bstep.szFilename == szOldFilename)
				pAsm->LoadFirstPass(pOldAsm);
			delete pOldAsm;
		}
		pAsm->SetOutput(&asmout);
		if (pAsm->LoadTabFile( pInfo->lpszTabFile))
			pAsm->Build( bstep.szFilename);
		bStopBuild = (pAsm->GetStatus() >= CAssembler::asmErrors);
	}

	UINT nResult = 0;
	if (pAsm) {
		pAsm->Finish();
		nResult = pAsm->GetStatus();
		delete pAsm;
	}
	::SendMessage(pInfo->hwndNotify, WM_USER_ASM_DONE, 0, 0L);
	CloseHandle(hEventAbort);
	delete pInfo;
	return nResult;	// return assembly status as thread result
}
*/
