// IntermediateFile.cpp: implementation of the CIntermediateFile class.
//
// Copyright (C) 2000 Jeremy Goetsch (jeremy@acz.org)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#include "stdafx.h"
#include "Asmide.h"
#include "IntermediateFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// String read/write

LPTSTR CIntermediateFile::ReadString( LPTSTR lpsz, UINT nMax )
{
	ASSERT(lpsz != NULL);
	ASSERT(AfxIsValidAddress(lpsz, nMax));
	char c;
	char *pos = lpsz;

	for (;;) {
		if (Read( &c, 1) < 1) {
			*pos = '\0';
			if (pos != lpsz)
				return( lpsz);
			else
				return( NULL);
		}

		*pos = c;
		pos++;
		if (c == '\n' || (UINT)(pos - lpsz) >= nMax) {
			*pos = '\0';
			return( lpsz);
		}
	}
}

BOOL CIntermediateFile::ReadString(CString& rString)
{
	rString = &afxChNil;	// empty string

	char c;
	for (;;) {
		if (Read( &c, 1) < 1) {
			if (!rString.IsEmpty())
				return( TRUE);
			else
				return( FALSE);
		}
		rString += c;
		if (c == '\n')
			return( TRUE);
	}
}

void CIntermediateFile::WriteString( LPCTSTR lpsz)
{
	ASSERT(lpsz != NULL);
	Write( lpsz, strlen(lpsz));
}


/////////////////////////////////////////////////////////////////////////////
// CIntermediateFile dump

#ifdef _DEBUG

void CIntermediateFile::Dump(CDumpContext& dc) const
{
	CMemFile::Dump(dc);
	dc << "Mapped files: " << m_FileList.GetSize();
	dc << "\nMapped lines: " << m_LineMap.GetSize();
}

#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// Line mapping routines

int CIntermediateFile::AddFileName( CString filename)
{
	return m_FileList.Add( filename);
}

BOOL CIntermediateFile::IsFileDefined( CString filename) const
{
	for (int i = 0; i < m_FileList.GetSize(); i++)
		if (m_FileList[i] == filename)
			return TRUE;
	return FALSE;
}

int CIntermediateFile::AddLine( int nFile, int nMappedLine)
{
	ASSERT( nFile <= m_FileList.GetUpperBound());

	SOURCELINEMAP linemap;
	linemap.nFile = (BYTE) nFile;
	linemap.nLine = nMappedLine;
	linemap.bSecondPass = FALSE;
	linemap.InstPos = NULL;
	return m_LineMap.Add( linemap);
}

void CIntermediateFile::SetLineSecondPass( int nAbsLine, BOOL bRequired)
{
	ASSERT( nAbsLine <= m_LineMap.GetUpperBound());
	m_LineMap[nAbsLine].bSecondPass = bRequired;
}

BOOL CIntermediateFile::GetLineSecondPass( int nAbsLine) const
{
	ASSERT( nAbsLine <= m_LineMap.GetUpperBound());
	return m_LineMap[nAbsLine].bSecondPass;
}

void CIntermediateFile::SetLineInstruction( int nAbsLine, POSITION pInstruction)
{
	ASSERT( nAbsLine <= m_LineMap.GetUpperBound());
	m_LineMap[nAbsLine].InstPos = pInstruction;
}

POSITION CIntermediateFile::GetLineInstruction( int nAbsLine) const
{
	ASSERT( nAbsLine <= m_LineMap.GetUpperBound());
	return m_LineMap[nAbsLine].InstPos;
}

CString CIntermediateFile::GetFilePath( int nAbsLine) const
{
	ASSERT( nAbsLine <= m_LineMap.GetUpperBound());
	return m_FileList[ m_LineMap[nAbsLine].nFile];
}

int CIntermediateFile::MapLine( int nAbsLine) const
{
	ASSERT( nAbsLine <= m_LineMap.GetUpperBound());
	return m_LineMap[nAbsLine].nLine;
}

int CIntermediateFile::GetNumLines() const
{
	return m_LineMap.GetSize();
}


/////////////////////////////////////////////////////////////////////////////
// Write contents to a text file

BOOL CIntermediateFile::SaveToFile( CString filename)
{
	CStdioFile out;
	if (!out.Open( filename, CFile::modeCreate | CFile::modeWrite))
		return FALSE;

	char szLine[MAX_LINE_LENGTH];
	while (ReadString(szLine, MAX_LINE_LENGTH))
		out.WriteString(szLine);
	return TRUE;
}
