// IntermediateFile.h: interface for the CIntermediateFile class
//                     used as output of the preprocessor
//
// Copyright (C) 2000 Jeremy Goetsch (jeremy@acz.org)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#if !defined(AFX_INTERMEDIATEFILE_H__CBAA0861_B534_11D1_A838_F8FFF7C89F75__INCLUDED_)
#define AFX_INTERMEDIATEFILE_H__CBAA0861_B534_11D1_A838_F8FFF7C89F75__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


// structure for mapping lines to original file and line num
struct SOURCELINEMAP
{
	BYTE nFile;			// containing file (index of m_FileList)
	int  nLine;			// line number in file
	BOOL bSecondPass;	// true if line needs second pass
	POSITION InstPos;	// POSITION of instruction in inst table linked list
						//  NULL if not instruction
};


class CIntermediateFile : public CMemFile  
{
public:
	// Stream-style reading/writing
	virtual LPTSTR ReadString( LPTSTR lpsz, UINT nMax );
	BOOL ReadString(CString& rString);
	virtual void WriteString( LPCTSTR lpsz);

public:
	// Source file line mapping
	int AddFileName( CString filename);
	BOOL IsFileDefined( CString filename) const;
	int AddLine( int nFile, int nMappedLine);
	void SetLineSecondPass( int nAbsLine, BOOL bRequired = TRUE);
	BOOL GetLineSecondPass( int nAbsLine) const;
	void SetLineInstruction( int nAbsLine, POSITION pInstruction);
	POSITION GetLineInstruction( int nAbsLine) const;
	CString GetFilePath( int nAbsLine) const;
	int MapLine( int nAbsLine) const;
	int GetNumLines() const;

	// write to a text file
	BOOL SaveToFile( CString filename);

protected:
	CStringArray m_FileList;
	CArray<SOURCELINEMAP,SOURCELINEMAP> m_LineMap;

public:
#ifdef _DEBUG
	virtual void Dump(CDumpContext& dc) const;
#endif
};

#endif // !defined(AFX_INTERMEDIATEFILE_H__CBAA0861_B534_11D1_A838_F8FFF7C89F75__INCLUDED_)
