// OnePassAsm.cpp: implementation of the one-pass assembler classes
//
// Copyright (C) 2000 Jeremy Goetsch (jeremy@acz.org)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#include "stdafx.h"
#include "Asmide.h"
#include "OnePassAsm.h"
#include "AsmOutputStream.h"
#include "IntermediateFile.h"
//#include "OutputEdit.h"
//#include "AsmStatsDlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// COnePassAssembler - base class for one-pass assembly operations

COnePassAssembler::COnePassAssembler(UINT nFlags, LPCSTR szDefines)
	: CAssembler(nFlags, szDefines)
{
}

void COnePassAssembler::Build( CString filename)
{
	ASSERT(m_pOutput != NULL);
	//m_pOutput->PrintInfo("Assembling " + filename + '\n');

	// preprocess
	Preprocess( filename);

	// first pass
	if (m_Status <= asmErrors)
		AssembleFirst();

	if (m_Status <= asmWarnings)
		Output(filename);
}

/////////////////////////////////////////////////////////////////////////////
// CStatsAssembler
//   Show the assembly statistics dialog
/*
CStatsAssembler::CStatsAssembler(UINT nFlags, LPCSTR szDefines)
	: COnePassAssembler(nFlags, szDefines)
{
}

void CStatsAssembler::Output(CString filename)
{
	CAsmStatsDlg dlg;
	dlg.m_stdlg.m_pLabelMap = m_pLabelMap;
	dlg.m_sFiles.Format("Assembled %i file(s)", m_numFiles);
	dlg.m_sNumLines.Format("%i Lines", m_pInterSrc->GetNumLines());
	dlg.m_sCodeSize.Format("Code Size:\t%i bytes", m_nCodeSize);
	dlg.m_sDataSize.Format("Data Size:\t%i bytes", m_nDataSize);
	dlg.m_sTotalSize.Format("Total Size:\t%i bytes", m_nCodeSize + m_nDataSize);
	dlg.m_sNumLabels.Format("# of Labels:\t%i", m_pLabelMap->GetCount());
	dlg.DoModal();
}
*/
/////////////////////////////////////////////////////////////////////////////
// CSymTableAssembler
//   Write include file

CSymTableAssembler::CSymTableAssembler(UINT nFlags, LPCSTR szDefines)
	: COnePassAssembler(nFlags, szDefines)
{
}

void CSymTableAssembler::Output( CString filename)
{
	CString outname;

	if (filename.Find('.') != -1)
		outname = filename.Left(filename.ReverseFind('.')) + ".inc";
	else
		outname = filename + ".inc";

	CStdioFile out;
	if (!out.Open( outname, CFile::modeCreate | CFile::modeWrite)) {
		m_pOutput->PrintInfo("Cannot create file " + filename);
		m_Status = asmErrors;
		return;
	}

	CString label, flentry;
	LabelDef lbldef;
	POSITION pos = m_pLabelDisp->GetStartPosition();
	while (pos != NULL) {
		m_pLabelDisp->GetNextAssoc( pos, label, lbldef);
		flentry.Format("%s = $%04x\n", label, lbldef.address);
		out.WriteString(flentry);
	}
	out.Close();
}
