// TwoPassAsm.cpp: implementation two-pass assembler derived classes
//
// Copyright (C) 2000 Jeremy Goetsch (jeremy@acz.org)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//


#include "stdafx.h"
#include "Asmide.h"
#include "TwoPassAsm.h"
#include "AsmOutputStream.h"
#include "IntermediateFile.h"
#include "Resource.h"
//#include "OutputEdit.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// dll function pointer type
typedef int (*SAVEFUNCPTR)(BYTE *, UINT, LPCSTR, LPSTR);

/////////////////////////////////////////////////////////////////////////////
// CTwoPassAssembler construction

CTwoPassAssembler::CTwoPassAssembler(UINT nFlags, LPCSTR szDefines)
	: CAssembler(nFlags, szDefines)
{
}

/////////////////////////////////////////////////////////////////////////////
// Two pass build function

void CTwoPassAssembler::Build( CString filename)
{
	ASSERT(m_pOutput != NULL);
	m_pOutput->PrintInfo("Assembling " + filename + '\n');

	// preprocess
	Preprocess( filename);

	// first pass
	if (m_Status <= asmErrors)
		AssembleFirst();

	// second pass
	CMemFile obj;
	if (m_Status <= asmWarnings)
		AssembleMain( obj);

	// write file with plugin
	if (m_Status <= asmWarnings) {
		obj.SeekToBegin();
		Output(filename, obj);
	}
}

//////////////////////////////////////////////////////////////////////
// CPluginAssembler construction/destruction

CPluginAssembler::CPluginAssembler(UINT nFlags, LPCSTR szPlugin, LPCSTR szDefines)
	: CTwoPassAssembler(nFlags, szDefines)
{
	m_strPlugin = szPlugin;
	if (m_strPlugin.GetLength() < 4 || m_strPlugin[m_strPlugin.GetLength() - 4] != '.')
		m_strPlugin += ".dll";
}

/////////////////////////////////////////////////////////////////////////////
// CPluginAssembler Output function - write file using plugin

void CPluginAssembler::Output(CString filename, CMemFile &fData)
{
	CString fn;
	char szExt[5];
	if (filename.Find('.') != -1)
		fn = filename.Left(filename.ReverseFind('.'));
	else
		fn = filename;

	HINSTANCE hinstLib = LoadLibrary(m_strPlugin);
	if (hinstLib != NULL) {
		SAVEFUNCPTR SavePrgmFile = (SAVEFUNCPTR)GetProcAddress(hinstLib, "SavePrgmFile");
		if (SavePrgmFile != NULL) {
			UINT nSize = fData.GetLength();
			BYTE *pData = fData.Detach();
			SavePrgmFile(pData, nSize, fn, szExt);
			delete [] pData;
		}
		else {
			cout << "Plugin file \"" << (LPCSTR)m_strPlugin << "\" does not exist or is an invalid file." << endl;
			exit(1);
		}
		FreeLibrary(hinstLib);
	}
	else {
		cout << "Plugin file \"" << (LPCSTR)m_strPlugin << "\" does not exist or is an invalid file." << endl;
		exit(1);
	}
}


//////////////////////////////////////////////////////////////////////
// CHexAssembler construction/destruction

CHexAssembler::CHexAssembler(UINT nFlags, LPCSTR szDefines)
	: CTwoPassAssembler(nFlags, szDefines)
{
}

//////////////////////////////////////////////////////////////////////
// CHexAssembler Output function - write hex listing to text file

void CHexAssembler::Output(CString filename, CMemFile& fData)
{
	CString outname;
	if (filename.Find('.') != -1)
		outname = filename.Left(filename.ReverseFind('.')) + ".txt";
	else
		outname = filename + ".txt";

	CStdioFile out;
	if (!out.Open( outname, CFile::modeCreate | CFile::modeWrite)) {
		m_pOutput->PrintInfo("Cannot create file " + filename);
		m_Status = asmErrors;
		return;
	}

	CString strOut;
	out.WriteString( out.GetFileTitle() + " - Hex Listing\n");
	strOut.Format( "%d bytes total\n\n", m_nCodeSize + m_nDataSize);
	out.WriteString( strOut);

	BYTE bval;
	UINT nCount = 0;
	while ( fData.Read( &bval, 1) == 1) {
		if (nCount >= 4) {
			out.WriteString("\n");
			nCount = 0;
		}
		nCount++;
		strOut.Format( "%02X ", bval);
		out.WriteString( strOut);
	}
}