// cz80.cpp : Defines the entry point for the console application.
//
// Copyright (C) 2001 Jonah Cohen (jonah@jonah.ticalc.org)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#include "stdafx.h"
#include "cz80.h"
#include "OnePassAsm.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CAsmideApp theApp;

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		return 1;
	}

	theApp.InitInstance();

	bool symbol = false, hex = false, plugin = false, invalid = argc < 2;
	CString pluginname;

	for(int i=1; i<argc-1 && !invalid; i++)
	{
		if(argv[i][0] != '-')
			invalid = true;
		else if(argv[i][1] == 'p')
		{
			if(i < argc-2)
			{
				plugin = true;
				pluginname = argv[++i];
			}
			else
				invalid = true;
		}
		else if(argv[i][1] == 's')
			symbol = true;
		else if(argv[i][1] == 'h')
			hex = true;
		else
			invalid = true;
	}

	if(invalid)
	{

		cout << "cz80 compiler by Jonah Cohen\n"
			 << "based on Assembly Studio 8x 4.0 Compiler by Jeremy Goetsch\n"
			 << "\n"
			 << "Syntax:\tcz80 [options] source.asm\n"
			 << "\n"
			 << "Options:\n"
			 << "\t-p plugin\t\tCompile with specific plugin\n"
			 << "\t-s\t\t\tOutput symbol table\n"
			 << "\t-h\t\t\tHex output\n";
		return 1;
	}

	SetCurrentDirectory(GetDirFromPath(argv[argc-1]));

	SingleAsmThreadInfo *pInfo = new SingleAsmThreadInfo;
	pInfo->lpszPath = argv[argc-1];

	if(symbol)
		pInfo->pAsm = new CSymTableAssembler(GetOptions()->m_asmErrorChk);
	else if(hex)
		pInfo->pAsm = new CHexAssembler(GetOptions()->m_asmErrorChk);
	else
		pInfo->pAsm = new CPluginAssembler(GetOptions()->m_asmErrorChk,
										   (plugin)?pluginname:GetOptions()->m_asmPluginFile);

	SingleAsmThreadProc(pInfo);

	return 0;
}


